body {
    font-size: small;
    font-family:Verdana,sans-serif;
    background-color: #ffffff;
}

#header {
    background-color: #294172;
    background: -moz-linear-gradient(top, #456dbf, #294172, #1c2c4c);
    background: -webkit-gradient(linear, left top, left bottom, from(#456dbf), to(#1c2c4c), color-stop(0.5, #294172));
    color: #ffffff;
    border: 1px solid #444444;
    border-bottom: none;
    -moz-border-radius-topleft: 0.5em;
    -moz-border-radius-topright: 0.5em;
    -webkit-border-top-left-radius: 0.5em;
    -webkit-border-top-right-radius: 0.5em;
    -khtml-border-top-left-radius: 0.5em;
    -khtml-border-top-right-radius: 0.5em;
    padding: 0.4em;
    font-weight: bold;
}

#header a {
    font-weight: normal;
    color: #ffffff;
}

#header #login {
    font-size: 85%;
    float: right;
}

#menu {
    border: 1px solid #8c8f91;
    border-top: none;
    -moz-border-radius-bottomright: 0.5em;
    -moz-border-radius-bottomleft: 0.5em;
    -webkit-border-bottom-left-radius: 0.5em;
    -webkit-border-bottom-right-radius: 0.5em;
    -khtml-border-bottom-left-radius: 0.5em;
    -khtml-border-bottom-right-radius: 0.5em;
    padding: 0.1em 0.3em 0.1em 0.3em;
    background: -moz-linear-gradient(top, #fefefe, #ececed, #c8c8ca);
    background: -webkit-gradient(linear, left top, left bottom, from(#fefefe), to(#d8d8da), color-stop(0.6, #ececed));
}

#menu a {
    font-size: 85%;
    text-decoration: none;
    margin: 0.2em;
    color: #000000;
}

#menu a.active {
    color: #9e292b;
    color: #3C6EB4;
}

#menu a:hover {
    text-decoration: underline;
}

#submenu {
    border-top: 1px solid #8c8f91;
}

#content #error_message, #content #info_message {
    border: 3px solid;
    padding: 0.3em;
    margin: 1em 0em 1em 0em;
}

#content #error_message {
    border-color: #cc0000;
    background-color: #ffcccc;
}

#content #info_message {
    border-color: #00cc00;
    background-color: #ccffcc;
}

#footer {
    border: 1px solid #8c8f91;
    -moz-border-radius: 0.5em;
    -webkit-border-radius: 0.5em;
    -khtml-border-radius: 0.5em;
    font-size: 75%;
    text-align: center;
    padding: 0.2em;
    margin-top: 2em;
}

pre.log {
    border: 1px solid #d0d0d0;
    overflow: auto;
    min-height: 10em;
}

table {
    border-collapse: collapse;
    margin-top: 1em;
    margin-bottom: 1em;
    font-size: 0.9em;
}

td, th {
    padding: 0.3em 0.7em 0.3em 0.7em;
    border: 1px solid #d0d0d0;
}

/* LIST TABLE */

table.list th {
    color: black;
    text-align: left;
    font-weight: bold;
    border: 1px solid #aaa;
    background-color: #d8d8da;
    background: -moz-linear-gradient(top, #fefefe, #ececed, #d8d8da);
    background: -webkit-gradient(linear, left top, left bottom, from(#fefefe), to(#d8d8da), color-stop(0.6, #ececed));
}

table.list tr:nth-child(even) {
    background-color: #f5f5f5;
}

/* DETAILS TABLE */

table.details {
    border-collapse: separate;
    margin-top: 1em;
    margin-bottom: 1em;
    border: none;
    font-size: 0.9em;
}

table.details td, table.details th {
    border: none;
}

table.details td {
    border-bottom: 1px solid #d0d0d0;
    min-width: 20em;
}

table.details th {
    width: 10em;
    -moz-border-radius-topleft: 0.8em;
    -moz-border-radius-bottomleft: 0.8em;
    -webkit-border-top-left-radius: 0.8em;
    -webkit-border-bottom-left-radius: 0.8em;
    -khtml-border-top-left-radius: 0.8em;
    -khtml-border-bottom-left-radius: 0.8em;
    text-align: right;
    font-weight: bold;
    background-color: #d8d8da;
    background: -moz-linear-gradient(left, #f0f0f0, #ececed, #d8d8da);
    background: -webkit-gradient(linear, left top, right top, from(#f0f0f0), to(#d8d8da), color-stop(0.6, #ececed));
}

/* PAGINATOR */

.paginator {
    clear: both;
    float: right;
    margin: -0.5em 0em -0.5em 0em;
}

.paginator img {
    vertical-align: middle;
    position: relative;
    top: -0.15em;
    border: none;
}

/* TASK STATES */

.FREE, .ASSIGNED, .OPEN  {
    color: blue;
}

.CLOSED  {
    color: green;
}

.FAILED {
    color: red;
}

.INTERRUPTED, .CANCELED {
    color: orange;
}
