{% extends "base.html" %}
{% load i18n %}

{% block content %}
<h2>{% trans 'Channel' %} #{{ channel.id }}: {{ channel.name }}</h2>

<h3>{% trans 'Details' %}</h3>
<table class="details">
  <tr>
    <th>{% trans "ID" %}</th>
    <td>{{ channel.id }}</td>
  </tr>
  <tr>
    <th>{% trans "Name" %}</th>
    <td>{{ channel.name }}</td>
  </tr>
</table>

<h3>{% trans 'Workers' %}</h3>
{% include "worker/list_include.html" %}

{% endblock %}
